﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Serialization;

namespace tx_reporting_expensereport
{
    // wrapping business object that holds the seperate expense report data
    [XmlType("expenses", IncludeInSchema = true)]
    public class Expenses : List<Expense>
    {
        [XmlElement("expense")]
        public List<Expense> expenses { get; set; }
    }

    [XmlType("expense", IncludeInSchema = true)]
    public class Expense
    {
        [XmlElement("purpose")]
        public string Purpose { get; set; }

        [XmlElement("statement")]
        public string Statement { get; set; }

        [XmlElement("pay_period_start")]
        public DateTime PayPeriodStart { get; set; }

        [XmlElement("pay_period_end")]
        public DateTime PayPeriodEnd { get; set; }

        [XmlElement("employee")]
        public Employee Employee { get; set; }

        [XmlElement("lineitem")]
        public List<LineItem> LineItems { get; set; }

        [XmlElement("advances")]
        public double Advances { get; set; }

        public double Subtotal
        {
            get
            {
                double dTotal = 0;

                foreach (LineItem item in this.LineItems)
                {
                    dTotal += item.line_total;
                }

                return dTotal;
            }
        }

        public double Total
        {
            get
            {
                double dTotal = 0;

                foreach (LineItem item in this.LineItems)
                {
                    dTotal += item.line_total;
                }

                return dTotal - this.Advances;
            }
        }

        public double total_housing
        {
            get
            {
                double dTotal = 0;

                foreach (LineItem item in this.LineItems)
                {
                    dTotal += item.Housing;
                }

                return dTotal;
            }
        }

        public double total_transport
        {
            get
            {
                double dTotal = 0;

                foreach (LineItem item in this.LineItems)
                {
                    dTotal += item.Transport;
                }

                return dTotal;
            }
        }

        public double total_fuel
        {
            get
            {
                double dTotal = 0;

                foreach (LineItem item in this.LineItems)
                {
                    dTotal += item.Fuel;
                }

                return dTotal;
            }
        }

        public double total_meal
        {
            get
            {
                double dTotal = 0;

                foreach (LineItem item in this.LineItems)
                {
                    dTotal += item.Meal;
                }

                return dTotal;
            }
        }

        public double total_entertainment
        {
            get
            {
                double dTotal = 0;

                foreach (LineItem item in this.LineItems)
                {
                    dTotal += item.Entertainment;
                }

                return dTotal;
            }
        }

        public double total_misc
        {
            get
            {
                double dTotal = 0;

                foreach (LineItem item in this.LineItems)
                {
                    dTotal += item.Misc;
                }

                return dTotal;
            }
        }
    }

    public class Employee
    {
        [XmlElement("name")]
        public string Name { get; set; }

        [XmlElement("firstname")]
        public string FirstName { get; set; }

        [XmlElement("department")]
        public string Department { get; set; }

        [XmlElement("position")]
        public string Position { get; set; }

        [XmlElement("manager")]
        public string Manager { get; set; }

        [XmlElement("number")]
        public string Number{ get; set; }

        [XmlElement("ssn")]
        public string SSN { get; set; }
    }

    public class LineItem
    {
        [XmlElement("date")]
        public DateTime Date { get; set; }

        [XmlElement("account")]
        public string Account { get; set; }

        [XmlElement("description")]
        public string Description { get; set; }

        [XmlElement("housing")]
        public double Housing { get; set; }

        [XmlElement("transport")]
        public double Transport { get; set; }

        [XmlElement("fuel")]
        public double Fuel { get; set; }

        [XmlElement("meal")]
        public double Meal { get; set; }

        [XmlElement("entertainment")]
        public double Entertainment { get; set; }

        [XmlElement("misc")]
        public double Misc { get; set; }

        public double line_total
        {
            get
            {
                return
                    this.Housing +
                    this.Transport +
                    this.Fuel +
                    this.Meal +
                    this.Entertainment +
                    this.Misc;
            }
        }
    
    }
}
